<?php

/*

PBSWeb: A Web-based Interface to PBS
Copyright (C) 2000, 2001  Paul Lu

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

.......................
Please leave this copyright notice intact.
See http://www.gnu.org/copyleft/gpl.html for further details

Contact information:
	Paul Lu
	Assistant Professor
	Dept. of Computing Science
	Athabasca Hall
	University of Alberta
	Edmonton, Alberta, T6G 2E8
	Canada

	E-mail:         paullu@cs.ualberta.ca
	Web:            http://www.cs.ualberta.ca/~paullu/

Other contributors to PBSWeb (as of July 2001)
	George Ma, Victor Salamon, Christopher Pinchak, Yaling Pei,
	Jonathan Schaeffer, Duane Szafron
*/

# --------------------------------------------------------------------------------------------
# Vick 20001221 contains database related functions in php
# --------------------------------------------------------------------------------------------
# { REVISIONS BEGIN
#   Chris 20010518 DB_
#   Vick 20010204 DB_submit_program has a new argument: path
#   Vick 20010130 DB_submit_job superseeds DB_submit_job_script
#   Vick 20010127 permanent connections can be reset by "apachectl graceful" or "killall -USR1 httpd"
#   Vick 20010127 introducing detailed debugging, permanent connections and DB_ naming
#   Vick 20010111 connection with the database is done through full connect string
#   Vick 20010110 finished auths, login and authPage 
#   Vick 20010108 finished auth according to protocol
#   Vick 20001221 created
# } REVISIONS END  
# --------------------------------------------------------------------------------------------

# --------------------------------------------------------------------------------------------
# INCLUDES
# --------------------------------------------------------------------------------------------
include_once("timeutils.php");
include_once("utils.php");
include_once("PBSWebConstants.php");
include_once("dbg.php");
# --------------------------------------------------------------------------------------------

# --------------------------------------------------------------------------------------------
# { GLOBALS BEGIN 

# Database name
# MODIFY ME
$PBSDatabaseName="pbsweb_release";
# MODIFY ME
$PBSDatabaseUser="postgres";
# No password currently because both Apache/PHP and DB run as same user
$PBSDatabasePassword="";
$PBSDatabaseConnectString="dbname=$PBSDatabaseName user=$PBSDatabaseUser password=$PBSDatabasePassword";

# Correct authorization prefix
$PBSAuthorizationPrefix="AUTH_SUCC:";

# } GLOBALS END   
# --------------------------------------------------------------------------------------------

# --------------------------------------------------------------------------------------------
# { FUNCTIONS BEGIN 

# returns "$PBSAuthorizationPrefix:pbs_sessionkey" on success, error string otherwise
function DB_auth($username, $password)
{
   global $PBSDatabaseConnectString;   
   global $PBSAuthorizationPrefix;

   pbsdbg(__FILE__, __LINE__,"pg_pConnect($PBSDatabaseConnectString);");
   $connection = pg_pConnect ($PBSDatabaseConnectString);
   if(!$connection)
   { return get_debugged()."Here is pg_errormessage()".pg_errormessage($connection); }  

   $querystr="select * from users where name = '$username';";
   pbsdbg(__FILE__, __LINE__,$querystr);
   $result=pg_exec($connection, $querystr);
   if(!$result)
   { 
     pg_close($connection);
     return get_debugged(). "Here is pg_errormessage()".pg_errormessage($connection); 
   }
   else
   {
      $nrrows=pg_numrows($result);
      if(-1==$nrrows)
      { 
        pg_close($connection);
        return "pg_numrows failed. Here is pg_errormessage()".pg_errormessage($connection); 
      }

      if(0==$nrrows)
      { 
        pg_close($connection);
        return "\nNo user by that name ($username) ! <p>"; 
      }
      else
      {
        $arr = pg_fetch_array ($result, 0);
        if($password == $arr["password"])
        { 
          # Old version: return "Authentification succeeded !"; 
          # New: return session key; first, generate it
          $timenow=getmicrotime();
          $sessionkey=$username.$password.$timenow;

	  // Chris 20010518: Added acctime initialization.
	  $nowtime = time();
          $querystr="insert into sessions (id,username,gentime,acctime,valid) 
                            values ('$sessionkey','$username', timestamp 'now','$nowtime','1');";
          pbsdbg(__FILE__, __LINE__,$querystr);
          $result=pg_exec($connection, $querystr); 
          if(!$result)
          {
             pg_close($connection);
             return get_debugged()."pg_errormessage()".pg_errormessage($connection);
          }

          return "$PBSAuthorizationPrefix$sessionkey"; 
        }
        else
        { 
          pg_close($connection);
          return "Authentification FAILED !"; 
        }
      }
   }
}

# returns "$PBSAuthorizationPrefix:pbs_sessionkey" on success, error string otherwise
function DB_auths($sessionkey)
{
   global $PBSDatabaseConnectString;   
   global $PBSAuthorizationPrefix;

   pbsdbg(__FILE__, __LINE__,"pg_pConnect($PBSDatabaseConnectString);");
   $connection = pg_pConnect ($PBSDatabaseConnectString);
   if(!$connection)
   { return "pg_pConnect failed. Here is pg_errormessage()".pg_errormessage($connection); }  

   $querystr="select * from sessions where id = '$sessionkey';";
   pbsdbg(__FILE__, __LINE__,$querystr);
   $result=pg_exec($connection, $querystr);
   if(!$result)
   { 
     pg_close($connection);
     return get_debugged()."pg_errormessage()".pg_errormessage($connection); 
   }
   else
   {
      $nrrows=pg_numrows($result);
      if(-1==$nrrows)
      { 
        pg_close($connection);
        return "pg_numrows failed. Here is pg_errormessage()".pg_errormessage($connection); 
      }

      if(0==$nrrows)
      { 
        pg_close($connection);
        return "\nNo sessionkey of this value: $sessionkey !<p>"; 
      }
      else
      {
        if(1<$nrrows)
        {
          pg_close($connection);
          return "\nDatabase configuration error. Field *id* is not primary key ! <p>"; 
        }
        else {
	  // Chris 20010518: Check if the last acctime was within 2 hours of
	  // now.  If not, then it has expired.
	  // Chris 20010604: Now we just expire the session rather than remove
	  // it entirely.
	  $arr = pg_fetch_array($result, 0);
	  // Make sure the session was valid on the last page.
	  if ($arr['valid'] == 1)
	    if ($arr['acctime'] < time() - 7200) {
	      pg_close($connection);
	      // remove the session entry from the database.
	      DB_expireSession($sessionkey);
	      // I'm not sure which of these actually kill the session, so
	      // they're both here.
	      $PHPSESSID = 0;
	      // Destroy the PHP session once we know that the database session
	      // has expired.
	      session_destroy();
	      return "\nSession has expired due to inactivity.  " . 
	        "Please <a href=\"/\">login</a> again.";
	    }
	    else
              return "$PBSAuthorizationPrefix$sessionkey";
	  // Here the session has already expired.
	  else
	    return "\nSession has previosly expired.  Please <a href=\"/\"> " .
	      "login</a> again.";
	}
      }
   }
}

# returns the value of the sessionkey, given the result of auth and auths
function DB_getSessionKey($authorization)
{
  global $PBSAuthorizationPrefix;
  
  $rez=strpos("Q".$authorization,$PBSAuthorizationPrefix);

  if((FALSE==$rez) || (1!=$rez))
  { return FALSE; }
  else
  { return substr($authorization, strlen($PBSAuthorizationPrefix)); }
}

# given a username and password, calls auth; if that fails, prints error page and terminates script
# if does not fail, returns sessionkey
function DB_login($username, $password)
{
   $authorization=DB_auth($username,$password);
   $sessionkey=DB_getSessionKey($authorization);
   if(FALSE==$sessionkey)
   {
     # Vick 20010109 for George: make dispErrorPage() it fancier ... its in utils.php
     # $temp = "$authorization$username$password";
     dispErrorPageAutoHelp($authorization);
     die();
   }
   return $sessionkey;
}

# given a sessionkey, calls auths; if that fails, prints error page and terminates script
# if does not fail, returns sessionkey
function DB_authPage($sessionkey)
{
   global $PBSDatabaseConnectString;

   $authorization=DB_auths($sessionkey);
   $sessionkey=DB_getSessionKey($authorization);
   if(FALSE==$sessionkey)
   {
     # Vick 20010109 for George: make dispErrorPage() it fancyer ... its in utils.php
     dispErrorPageAutoHelp($authorization);
     die();
   }
   // Chris 20010518: now the acctime is updated to the current time.
   $nowtime = time();
   $update_query = "update sessions set acctime = '$nowtime' where".
     " id = '$sessionkey';";
   $connection = pg_connect($PBSDatabaseConnectString);
   pbsdbg(__FILE__, __LINE__,$update_query);
   $result = pg_exec($connection, $update_query);
   if(!$result) { 
     pg_close($connection);
     $PBSWebErrMsg="Failed updating session access time.";
     pbsdbg(__FILE__, __LINE__,"pg_errormessage()".
       pg_errormessage($connection)); 
     return PBSFALSERET;
   }
   return $sessionkey;
}

# should return the list of machines the user may have access to
# in this version returns the list of all machines
function DB_get_machines_list($username)
{
   global $PBSDatabaseConnectString;   
   global $PBSWebErrMsg;

   pbsdbg(__FILE__, __LINE__,"pg_pConnect($PBSDatabaseConnectString);");
   $connection = pg_pConnect($PBSDatabaseConnectString);
   if(!$connection)
   { 
     $PBSWebErrMsg="Failed retrieving machines list.";
     pbsdbg(__FILE__, __LINE__,"pg_errormessage()".pg_errormessage($connection)); 
     return PBSFALSERET;
   }

   $querystr="select name from hosts;";
   pbsdbg(__FILE__, __LINE__,"pg_pConnect($PBSDatabaseConnectString);");
   $result=pg_exec($connection, $querystr);

   if(!$result)
   { 
     pg_close($connection);
     $PBSWebErrMsg="Failed retrieving machines list.";
     pbsdbg(__FILE__, __LINE__,"pg_errormessage()".pg_errormessage($connection)); 
     return PBSFALSERET;
   }
   else
   {
      $nrrows=pg_numrows($result);
      if(-1==$nrrows)
      { 
        pg_close($connection);
        $PBSWebErrMsg="Failed to retrieve machines list."; 
        pbsdbg(__FILE__, __LINE__,"pg_errormessage()".pg_errormessage($connection)); 
        return PBSFALSERET;
      }

      if(0==$nrrows)
      { 
        pg_close($connection);
        $PBSWebErrMsg="No hosts defined !"; 
        pbsdbg(__FILE__, __LINE__,"pg_errormessage()".pg_errormessage($connection)); 
        return PBSFALSERET;
      }
      else
      {
        for($i=0;$i<$nrrows;$i++)
        { 
          $arr = pg_fetch_row ($result, $i);
          $rez[$i]=$arr[0];
        }
      }
   }
   return $rez;
}

# inserts a tuple in the programs table
function DB_submit_program($username, $sessionkey, $programname, $filename, $path, $machinename)
{
   global $PBSDatabaseConnectString;   
   global $PBSWebErrMsg;

   pbsdbg(__FILE__, __LINE__,"pg_pConnect($PBSDatabaseConnectString);");
   $connection = pg_pConnect ($PBSDatabaseConnectString);
   if(!$connection)
   { 
     $PBSWebErrMsg="pg_pConnect failed. Here is pg_errormessage()".pg_errormessage($connection); 
     return PBSFALSERET;
   }

   $querystr="insert into programs (username, program, filename, path, host,
                                    sessionkey,upload_time,deleted) 
                     values ('$username','$programname', '$filename', '$path', '$machinename',
                             '$sessionkey',timestamp 'now','0');";
   pbsdbg(__FILE__, __LINE__,$querystr);
   $result=pg_exec($connection, $querystr);
   if(!$result)
   { 
     pg_close($connection);
     $PBSWebErrMsg="Failed submitting program."; 
     pbsdbg(__FILE__, __LINE__,"pg_errormessage()".pg_errormessage($connection)); 
     return PBSFALSERET;
   }
   return 1;
}

# given a username and a host name, returns an array of programs submitted to that machine.
function DB_get_programs_list($username, $machinename)
{
   global $PBSDatabaseConnectString;   
   global $PBSWebErrMsg;

   pbsdbg(__FILE__, __LINE__,"pg_pConnect($PBSDatabaseConnectString);");
   $connection = pg_pConnect ($PBSDatabaseConnectString);
   if(!$connection)
   { 
     $PBSWebErrMsg="pg_pConnect failed. Here is pg_errormessage()".pg_errormessage($connection); 
     return PBSFALSERET;
   }

   $querystr="select program from programs where 
              username='$username' and host='$machinename';";
   pbsdbg(__FILE__, __LINE__,$querystr);
   $result=pg_exec($connection, $querystr);
   if(!$result)
   { 
     pg_close($connection);
     $PBSWebErrMsg="Failed getting program list."; 
     pbsdbg(__FILE__, __LINE__,"pg_errormessage()".pg_errormessage($connection)); 
     return PBSFALSERET;
   }
   else
   {
      $nrrows=pg_numrows($result);
      if(-1==$nrrows)
      { 
        pg_close($connection);
        $PBSWebErrMsg="Failed getting program list."; 
        pbsdbg(__FILE__, __LINE__,"pg_errormessage()".pg_errormessage($connection)); 
        return PBSFALSERET;
      }
      else
      {
        for($i=0;$i<$nrrows;$i++)
        { 
          $arr = pg_fetch_row($result, $i);
          $rez[$i]=$arr[0];
        }
      }
   }
   return $rez;
}

# returns the list of all jobs submitted by this user for this program on this host
function DB_get_job_list($username, $programname, $machinename)
{
   global $PBSDatabaseConnectString;   
   global $PBSWebErrMsg;

   pbsdbg(__FILE__, __LINE__,"pg_pConnect($PBSDatabaseConnectString);");
   $connection = pg_pConnect ($PBSDatabaseConnectString);
   if(!$connection)
   { 
     $PBSWebErrMsg="pg_pConnect failed. Here is pg_errormessage()".pg_errormessage($connection); 
     return PBSFALSERET;
   }

   $querystr="select job_name from submissions where 
              username='$username' and host='$machinename' 
              and program_name='$programname';";
   pbsdbg(__FILE__, __LINE__,$querystr);
   $result=pg_exec($connection, $querystr);
   if(!$result)
   { 
     pg_close($connection);
     $PBSWebErrMsg="Failed getting job list."; 
     pbsdbg(__FILE__, __LINE__,"pg_errormessage()".pg_errormessage($connection)); 
     return PBSFALSERET;
   }
   else
   {
      $nrrows=pg_numrows($result);
      if(-1==$nrrows)
      { 
        pg_close($connection);
        $PBSWebErrMsg="Failed getting job list."; 
        pbsdbg(__FILE__, __LINE__,"pg_errormessage()".pg_errormessage($connection)); 
        return PBSFALSERET;
      }
      else
      {
        for($i=0;$i<$nrrows;$i++)
        { 
          $arr = pg_fetch_row($result, $i);
          $rez[$i]=$arr[0];
        }
      }
   }
   return $rez;
}

# submits an entry in the submissions table, then one in the jobs table.
function DB_submit_job($username, $sessionkey, $programname, $machinename, $pbsjobid, $job)
{
   global $PBSDatabaseConnectString;   
   global $PBSWebErrMsg;

   $jobname=$job["jobname"];

   if((""==$jobname)||(strlen($jobname)>50))
   {
     $PBSWebErrMsg="Jobname should be non-null and less than 50 chars in length. I got $jobname.";
     return PBSFALSERET;
   }

   $jobscript=$job["jobscript"];
   if((""==$jobscript)||(2000<strlen($jobscript)))
   {
     $PBSWebErrMsg="The script should be non-empty and less than 2000 chars in length.";
     return PBSFALSERET;
   }

   $queue=$job["queue"];
   # Vick 20010220 the queue name can be an empty string
   if(50<strlen($queue))
   {
     $PBSWebErrMsg="The queue name should be less than 50 chars in length.";
     return PBSFALSERET;
   }

   $ncpus=$job["ncpus"];
   if(""==$ncpus)
   {
     $PBSWebErrMsg="Please enter a valid number of CPUs.";
     return PBSFALSERET;
   }

   $maxtime=$job["maxtime"];
   $tmp=split(":",$maxtime);
   if((count($tmp)!=3) || 
      (strlen($tmp[0])!=2) ||
      (strlen($tmp[1])!=2) ||
      (strlen($tmp[2])!=2))
   {
     $PBSWebErrMsg="Internal error. Time format should be hh:mm:ss, I got $maxtime.";
     return PBSFALSERET;
   }
   if(($tmp[1]<0) || ($tmp[1]>59) ||
      ($tmp[2]<0) || ($tmp[2]>59))
   {
     $PBSWebErrMsg="Internal error. Minutes and seconds should be [0,59], I got $maxtime.";
     return PBSFALSERET;
   }
   $maxtime_conv=$tmp[0]*60*60*1000+
                 $tmp[1]*60*1000+
		 $tmp[2]*1000;

   $merge=$job["merge"];

   # these are permitted to be null
   $in_here=$job["in_here"];
   $in_there=$job["in_there"];
   $out_here=$job["out_here"];
   $out_there=$job["out_there"];

   $mail_start=$job["mail_start"];
   if(""==$mail_start)
   { $mail_start='0'; }

   $mail_end=$job["mail_end"];
   if(""==$mail_end)
   { $mail_end='0'; }

   $mail_abort=$job["mail_abort"];
   if(""==$mail_abort)
   { $mail_abort='0'; }

   # this can not be null
   $mailaddress=$job["mailaddress"];
   if(""==$mailaddress)
   {
     $PBSWebErrMsg="Please enter a valid email address.";
     return PBSFALSERET;
   }

   $dbgmsg="\nHash contents:<br>";
   $dbgmsg.="\njobname=$jobname<br>";
   $dbgmsg.="\njobscript=$jobscript<br>";
   $dbgmsg.="\nqueue=$queue<br>";
   $dbgmsg.="\nncpus=$ncpus<br>";
   $dbgmsg.="\nmaxtime=$maxtime<br>";
   $dbgmsg.="\nmaxtime_conv=$maxtime_conv (actually this is local conversion of previous)<br>";
   $dbgmsg.="\nmerge=$merge<br>";
   $dbgmsg.="\nin_here=$in_here<br>";
   $dbgmsg.="\nin_there=$in_there<br>";
   $dbgmsg.="\nout_here=$out_here<br>";
   $dbgmsg.="\nout_there=$out_there<br>";
   $dbgmsg.="\nmail_start=$mail_start<br>";
   $dbgmsg.="\nmail_end=$mail_end<br>";
   $dbgmsg.="\nmail_abort=$mail_abort<br>";
   pbsdbg(__FILE__, __LINE__,$dbgmsg);

   pbsdbg(__FILE__, __LINE__,"pg_pConnect($PBSDatabaseConnectString);");
   $connection = pg_pConnect ($PBSDatabaseConnectString);
   if(!$connection)
   { 
     $PBSWebErrMsg="pg_pConnect failed. Here is pg_errormessage()".pg_errormessage($connection); 
     return PBSFALSERET;
   }

   # the value of the script id is based on sessionkey and current time
   $scriptid=getmicrotime().$sessionkey;

   $querystr="insert into jobs (jobscript, output,  script_id, queue, 
                                ncpus,  maxtime, merge, 
				in_here, in_there, out_here, out_there,
				mail_start, mail_end, mail_abort, mailaddress,
				delete)
                          values  ('$jobscript', '$output',  '$scriptid', '$queue', 
                                   '$ncpus',  '$maxtime_conv', '$merge', 
				   '$in_here', '$in_there', '$out_here', '$out_there',
				   '$mail_start', '$mail_end', '$mail_abort', '$mailaddress',
				   0);";
   pbsdbg(__FILE__, __LINE__,$querystr); 
   $result=pg_exec($connection, $querystr);
   if(!$result)
   { 
     pg_close($connection);
     $PBSWebErrMsg="Failed submitting script."; 
     pbsdbg(__FILE__, __LINE__,"pg_errormessage()".pg_errormessage($connection)); 
     return PBSFALSERET;
   }

   # Vick 20010213 if there's another job with the same job_name, overwrite it
   $querystr="update submissions set overwritten='1' 
              where username='$username' 
	      and   job_name='$jobname'
	      and   program_name='$programname'
	      and   host='$machinename';";
   pbsdbg(__FILE__, __LINE__, $querystr); 
   $result=pg_exec($connection,$querystr);
   if(!$result)
   { 
     pg_close($connection);
     $PBSWebErrMsg="Failed submitting script."; 
     pbsdbg(__FILE__, __LINE__,"pg_errormessage()".pg_errormessage($connection)); 
     return PBSFALSERET;
   }

   $querystr="insert into submissions (username, sessionkey, program_name, host,
                                       job_name,script_id,submit_time, overwritten) 
                     values ('$username','$sessionkey','$programname', '$machinename',
                             '$jobname','$scriptid',timestamp 'now','0');";
   pbsdbg(__FILE__, __LINE__, $querystr); 
   $result=pg_exec($connection,$querystr);

   if(!$result)
   { 
     pg_close($connection);
     $PBSWebErrMsg="Failed submitting script."; 
     pbsdbg(__FILE__, __LINE__,"pg_errormessage()".pg_errormessage($connection)); 
     return PBSFALSERET;
   }

   return 1;
}

# given an username, programname, hostname and a jobname, returns the actual submitted script
function DB_get_job($username, $programname, $machinename, $jobname)
{
   global $PBSDatabaseConnectString;
   global $PBSWebErrMsg;

   pbsdbg(__FILE__, __LINE__,"pg_pConnect($PBSDatabaseConnectString);");
   $connection = pg_pConnect ($PBSDatabaseConnectString);
   if(!$connection)
   {
     $PBSWebErrMsg="Failed getting script."; 
     pbsdbg(__FILE__, __LINE__,"pg_errormessage()".pg_errormessage($connection)); 
     return PBSFALSERET;
   }

   $querystr="select script_id from submissions
              where username='$username' and
                    program_name='$programname' and
                    host='$machinename' and
                    job_name='$jobname' and
                    overwritten='0';";
   pbsdbg(__FILE__, __LINE__, $querystr); 
   $result=pg_exec($connection, $querystr);

   if(!$result)
   {
     pg_close($connection);
     $PBSWebErrMsg="Failed getting script."; 
     pbsdbg(__FILE__, __LINE__,"pg_errormessage()".pg_errormessage($connection)); 
     return PBSFALSERET;
   }

   $nrrows=pg_numrows($result);
   if(-1==$nrrows)
   {
     pg_close($connection);
     $PBSWebErrMsg="Failed getting script."; 
     pbsdbg(__FILE__, __LINE__,"pg_errormessage()".pg_errormessage($connection)); 
     return PBSFALSERET;
   }

   if(1<$nrrows)
   {
     pg_close($connection);
     $PBSWebErrMsg="There's more than one script_id with this property in submissions.";
     pbsdbg(__FILE__, __LINE__,"There's more than one script_id with this property in submissions."); 
     return PBSFALSERET;
   }

   if(0==$nrrows)
   {
     pg_close($connection);
     $PBSWebErrMsg="There's no script_id with this property in submissions. <br>";
     pbsdbg(__FILE__, __LINE__,"There's no script_id with this property in submissions."); 
     return PBSFALSERET;
   }
        
   $arr = pg_fetch_row($result, 0);
   $scriptid=$arr[0];
  
   $querystr="select jobscript,script_id,queue,ncpus,maxtime,merge,
                     in_here, in_there, out_here, out_there, 
		     mail_abort, mail_start, mail_end, 
		     mailaddress, delete from jobs where script_id='$scriptid';"; 
   pbsdbg(__FILE__, __LINE__, $querystr); 
   $result=pg_exec($connection, $querystr);
   if(!$result)
   {
     pg_close($connection);
     $PBSWebErrMsg="Failed getting script."; 
     pbsdbg(__FILE__, __LINE__,"pg_errormessage()".pg_errormessage($connection)); 
     return PBSFALSERET;
   }

   $nrrows=pg_numrows($result);
   if(-1==$nrrows)
   {
     pg_close($connection);
     $PBSWebErrMsg="Failed getting script."; 
     pbsdbg(__FILE__, __LINE__,"pg_errormessage()".pg_errormessage($connection)); 
     return PBSFALSERET;
   }

   if(1!=$nrrows)
   {
     pg_close($connection);
     $PBSWebErrMsg="There's $nrrows script(s) with this property in jobs.";
     pbsdbg(__FILE__, __LINE__,"There's $nrrows script(s) with this property in jobs."); 
     return PBSFALSERET;
   }

  $arr = pg_fetch_array($result, 0);

  # transform time back in hh:mmss format
  $maxtime=$arr["maxtime"];
  $hh=floor($maxtime/(60*60*1000)); 
  $maxtime=$maxtime%(60*60*1000);
  $mm=floor($maxtime/(60*1000)); 
  $maxtime=$maxtime%(60*1000);
  $ss=floor($maxtime/(1000));

  if(2!=strlen($hh))
  { $hh="0".$hh; }

  if(2!=strlen($mm))
  { $mm="0".$mm; }

  if(2!=strlen($ss))
  { $ss="0".$ss; }

  $arr["maxtime"]="$hh:$mm:$ss";

  # Vick 20010221 queue sometimes gets returned as ' '
# FIXME
# Paul, July 18/01:
# Perhaps this should be $arr["queue"] = str_replace(" ","",$arr["queue"]);

  str_replace(" ","",$arr["queue"]);

  return $arr;
}

function DB_getUserEmail($username)
{
   global $PBSDatabaseConnectString;   
   global $PBSWebErrMsg;

   pbsdbg(__FILE__, __LINE__,"pg_pConnect($PBSDatabaseConnectString);");
   $connection = pg_pConnect ($PBSDatabaseConnectString);
   if(!$connection)
   { 
     $PBSWebErrMsg="pg_pConnect failed. Here is pg_errormessage()".pg_errormessage($connection); 
     return PBSFALSERET;
   }

   $querystr="select email from users where name='$username';";
   pbsdbg(__FILE__, __LINE__, $querystr); 
   $result=pg_exec($connection, $querystr);
   if(!$result)
   { 
     pg_close($connection);
     $PBSWebErrMsg="Failed getting user email."; 
     pbsdbg(__FILE__, __LINE__,"pg_errormessage()".pg_errormessage($connection)); 
     return PBSFALSERET;
   }

   $nrrows=pg_numrows($result);
   if(-1==$nrrows)
   { 
     pg_close($connection);
     $PBSWebErrMsg="Failed getting user email."; 
     pbsdbg(__FILE__, __LINE__,"pg_errormessage()".pg_errormessage($connection)); 
     return PBSFALSERET;
   }

   if(1!=$nrrows)
   {
     pg_close($connection);
     $PBSWebErrMsg="There's $nrrows users with this username."; 
     pbsdbg(__FILE__, __LINE__,"There's $nrrows users with this username."); 
     return PBSFALSERET;
   }

   $arr = pg_fetch_row($result, 0);
   $rez=$arr[0];

    return $rez;
}

#function added by George 2000/02/02

function DB_ProgramExists($username,$host,$program,$theDBName) {

  global $PBSDatabaseConnectString;
  $program_exists=FALSE;

  $connection = pg_Connect ($PBSDatabaseConnectString);
   
  if(!$connection) {
    echo "pg_Connect failed. Here is pg_errormessage()".pg_errormessage($connection);
    pg_close($connection);
  }  
  else {
    $result = pg_exec($connection, "select * from programs where username='$username' and program='$program' and host='$host';");
    $numrows = pg_numrows($result);
    if ($numrows>0) {
      $program_exists=TRUE;
    }
    pg_close($connection);
    
    if ($program_exists) {
      return TRUE;
    }
    else {
      return PBSFALSERET;
    }
  }
}

function DB_getProgramPath($username,$host,$program)
{
   global $PBSDatabaseConnectString;   
   global $PBSWebErrMsg;

   pbsdbg(__FILE__, __LINE__,"pg_pConnect($PBSDatabaseConnectString);");
   $connection = pg_pConnect ($PBSDatabaseConnectString);
   if(!$connection)
   { 
     $PBSWebErrMsg="pg_pConnect failed. Here is pg_errormessage()".pg_errormessage($connection); 
     return PBSFALSERET;
   }

   $querystr="select path from programs where username='$username' and program='$program' and host='$host';";
   pbsdbg(__FILE__, __LINE__, $querystr); 
   $result=pg_exec($connection, $querystr);
   if(!$result)
   { 
     pg_close($connection);
     $PBSWebErrMsg="Failed getting program path."; 
     pbsdbg(__FILE__, __LINE__,"pg_errormessage()".pg_errormessage($connection)); 
     return PBSFALSERET;
   }

   $nrrows=pg_numrows($result);
   if(-1==$nrrows)
   { 
     pg_close($connection);
     $PBSWebErrMsg="Failed getting program path."; 
     pbsdbg(__FILE__, __LINE__,"pg_errormessage()".pg_errormessage($connection)); 
     return PBSFALSERET;
   }

   if(1!=$nrrows)
   {
     pg_close($connection);
     $PBSWebErrMsg="There's $nrrows paths for this program."; 
     pbsdbg(__FILE__, __LINE__,"There's $nrrows paths for this program."); 
     return PBSFALSERET;
   }

   $arr = pg_fetch_row($result, 0);
   $rez=$arr[0];

    return $rez;
}

// Functions added by Chris on May 16, 2001.

// Function to check the password.  Returns PBSFALSERET on error or when != 1
// rows are returned by the query.  Otherwise, nothing is returned.
function db_check_passwd($username, $passwd)
{
  global $PBSDatabaseConnectString;
  $query = "select * from users where name = '$username' and  password = '$passwd';";
  $connection = pg_connect($PBSDatabaseConnectString);
  pbsdbg(__FILE__, __LINE__,$query);
  $result = pg_exec($connection, $query);
  if (!$result) {
    pg_close($connection);
    $PBSWebErrMsg="Failed fetching user password."; 
    pbsdbg(__FILE__, __LINE__,
      "pg_errormessage()".pg_errormessage($connection));
    return PBSFALSERET;
  }
  $numrows = pg_numrows($result);
  if ($numrows != 1)
    return PBSFALSERET;
}

// Function that actually updates the password to the new password.  Returns
// PBSFALSERET on error, nothing on success.  Name should be the primary key,
// so there should not be an issue with multiple updates.  Ensure that username
// is not "*", or there could potentially be problems.
function db_change_passwd($username, $oldpasswd, $newpasswd)
{
   global $PBSDatabaseConnectString;
   $sql_update = "update users set password = '$newpasswd' where name = '$username' and password = '$oldpasswd';";
   $connection = pg_connect($PBSDatabaseConnectString);
   pbsdbg(__FILE__, __LINE__,$sql_update);
   $result = pg_exec($connection, $sql_update);
   if (!$result) {
     pg_close($connection);
     $PBSWebErrMsg="Failed updating user password."; 
     pbsdbg(__FILE__, __LINE__,
       "pg_errormessage()".pg_errormessage($connection));
     return PBSFALSERET;
   }
   $numrows = pg_numrows($result);
   if ($numrows == -1) {
     pbsdgb(__FILE__, __LINE__, "pg_errormessage()" .
       pg_errormessage($connection));
     return PBSFALSERET;
   }
}

// Chris 20010522: Removes a given session key from the database, effectively
// expiring that session.
function DB_expireSession($sessionkey)
{
  global $PBSDatabaseConnectString;
  $connection = pg_connect($PBSDatabaseConnectString);
  if(!$connection)
   { 
     $PBSWebErrMsg="Failed on connection.";
     pbsdbg(__FILE__, __LINE__,"pg_errormessage()".pg_errormessage($connection)); 
     return PBSFALSERET;
   }
  $query = "update sessions set valid = '0' where id = '$sessionkey';";
  pbsdbg(__FILE__, __LINE__,$query);
  $result = pg_exec($connection, $query);
  if (!$result) {
     pg_close($connection);
     $PBSWebErrMsg="Failed deleting session from sessions table."; 
     pbsdbg(__FILE__, __LINE__,
       "pg_errormessage()".pg_errormessage($connection));
     return PBSFALSERET;
  }
}

// Chris 20010601: Function to get any field of the hosts table.
function DB_getHostInfo($host, $field)
{
  global $PBSDatabaseConnectString;
  $connection = pg_connect($PBSDatabaseConnectString);
  if(!$connection)
   { 
     $PBSWebErrMsg="Failed on connection.";
     pbsdbg(__FILE__, __LINE__,"pg_errormessage()".pg_errormessage($connection)); 
     return PBSFALSERET;
   }
  $query = "select $field from hosts where name = '$host';";
  pbsdbg(__FILE__, __LINE__,$query);
  $result = pg_exec($connection, $query);
  if (!$result) {
     pg_close($connection);
     $PBSWebErrMsg="Failed retrieving host command.";
     pbsdbg(__FILE__, __LINE__,
       "pg_errormessage()".pg_errormessage($connection));
     return PBSFALSERET;
  }
  $arr = pg_fetch_array($result, 0);
  return $arr[0];
}
# } FUNCTIONS END 
# --------------------------------------------------------------------------------------------
?>
